from    asteriskutils.setting import lang_pack
def lang(txt:str) -> str:
    '''
    翻译函数
    Args
        txt(str):需要翻译的文本
    Returns
        str:翻译后的文本
    '''
    if lang_pack.get(txt):
        return lang_pack[txt]
    else:
        return txt


import datetime,chinese_calendar

def is_business_day(somedate: datetime.date=datetime.datetime.now()) -> bool:
    '''
    判断是否是工作日
    Args
        somedate(datetime.date):要判断的日期，默认是今天
    Returns
        bool:是否是工作日
    '''
    return somedate.weekday() in (0,1,2,3,4) and not chinese_calendar.is_holiday(somedate)





from colorama import init
import logging.config,datetime

# 解决Window环境中打印颜色问题
init(autoreset=True)


def get_logger(log_type:int = 0)-> logging.Logger:
    '''
    根据AppConfig设置log基本配置，并返回logger对象
    Args
        log_type(int):日志类型，默认为0，0为信息日志，1为执行成功日志，2为调试日志，3为警告日志，4为错误日志
    Returns
        logger: 日志实例
    '''
    from asteriskutils.setting import AppConfig
    
    import os
    # 根据AppConfig的设置拼接日志文件名
    match log_type:
        case 0 | 1:
            fn = f"event-{datetime.datetime.now().strftime(AppConfig['log']['log_file_name'])}.log"
        case 2:
            fn = f"debug-{datetime.datetime.now().strftime(AppConfig['log']['log_file_name'])}.log"
        case 3:
            fn = f"warning-{datetime.datetime.now().strftime(AppConfig['log']['log_file_name'])}.log"
        case 4:
            fn = f"error-{datetime.datetime.now().strftime(AppConfig['log']['log_file_name'])}.log"
    

    # fn = f'{datetime.datetime.now().strftime(AppConfig['log']['log_file_name'])}.log'.format()
    # 使用join的目的是适配不同操作系统
    # 载入日志的配置文件
    logging.config.fileConfig(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'log.config'))
    logger = logging.getLogger('MainLogger')
    try:
        fh = logging.FileHandler(os.path.join(os.path.abspath(os.curdir),AppConfig['log']['log_path'],fn),'a','utf-8')
    except FileNotFoundError:
        try:
            os.mkdir(os.path.join(os.path.abspath(os.curdir),AppConfig['log']['log_path']))
            fh = logging.FileHandler(os.path.join(os.path.abspath(os.curdir),AppConfig['log']['log_path'],fn) )
        except:
            from asteriskutils.i18n import lang
            print(f"\033[31m{lang('failed_to_create_log_folder')}，请在调用log前，覆盖设置设置AppConfig['log']['log_path']到项目目录.\033[0m")   
            raise FileNotFoundError
        
    fh.setFormatter(logging.Formatter(AppConfig['log']['log_format']))
    logger.addHandler(fh)
    return logger


def success_print(txt,header=True,*args,**kwargs) -> None:
    '''
    成功信息打印，绿色
    Args
        txt(str):需要在屏幕输出的文字
        header(bool):是否打印开头的[成功信息]
        args(list):其他print的参数
        kwargs(dict):其他print的参数
    '''
    s = '[成功信息]' if header else ''
    print(f'\033[32m{s}{txt}\033[0m',*args,**kwargs)
    get_logger(1).info(f'{s}{txt}')
def error_print(txt,header=True,*args,**kwargs):
    '''
    错误信息打印，红色
    Args
        txt(str):需要在屏幕输出的文字
        header(bool):是否打印开头的[成功信息]
        args(list):其他print的参数
        kwargs(dict):其他print的参数
    '''
    s = '[错误信息]' if header else ''
    print(f'\033[31m{s}{txt}\033[0m',*args,**kwargs)
    get_logger(4).error(f'{s}{txt}',exc_info=True, stack_info=True,)

def iprint(txt,header:str='运行信息',*args,**kwargs) -> None:
    '''
    输出信息打印，蓝色
    Args
        txt(str):需要在屏幕输出的文字
        header(str):是否打印开头的[运行信息]
        args(list):其他print的参数
        kwargs(dict):其他print的参数
    '''
    s = f'[{header}]' if header else ''
    print(f'\033[36m{s}{txt}\033[0m',*args,**kwargs)
    if header:
        get_logger().info(f'{s}{txt}')
def wprint(txt:str,header:str='警告信息',*args,**kwargs) -> None:
    '''
    警告信息打印，黄色
    Args
        txt(str):需要在屏幕输出的文字
        header(str):是否打印开头的[警告信息]的文字，默认为'警告信息'
        args(list):其他print的参数
        kwargs(dict):其他print的参数
    '''
    s = f'[{header}]:' if header else ''
    print(f'\033[33m{s}{txt}\033[0m',*args,**kwargs)
    get_logger(3).warning(f'{s}{txt}')

def dprint(txt,header:str='调试信息',*args,**kwargs) -> None:
    '''
    当配置文件中AppConfig的debug属性为true时打印，否则不打印
    按照以下格式:
    [调试信息] [{参数名/函数名}] :{value}
    Args
        txt(str):需要在屏幕输出的文字
        header(str):是否打印开头的[调试信息]
        args(list):其他print的参数
        kwargs(dict):其他print的参数
    '''
    from asteriskutils.setting import AppConfig
    import inspect
    if AppConfig['debug']:
        s = f"[{header}]:" if header else ''
        txt_name = 'Unknown' # 默认值
        txt_name = inspect.stack()[1][4][0].split("(")[1].split(")")[0] # 获取参数名称或者函数名称        
        print(f'\033[35m{s}[{txt_name}] => {txt}\033[0m',*args,**kwargs)
        get_logger(2).debug(f'{s}[{txt_name}] => {txt}')


'''
把的smtp进行再次封装，以便简化操作
'''
from email.mime.multipart import MIMEMultipart
from email.mime.base import MIMEBase
from email import encoders  
import smtplib
from email.header import Header
from email.mime.text import MIMEText
from email.utils import parseaddr, formataddr
from asteriskutils.error import EmailNotValidError
import re
from asteriskutils.tools import error_print, wprint

class AsteriskMailMsg():
    '''
    为Submarine封装邮件消息
    '''
    

    def __init__(self,from_eamil_address,from_name:str='系统管理员') -> None:
        '''
        初始化
        Args:
            from_email_address(str):发送者的邮件地址
        '''
        self.__to_addr = []
        self.__from_addr = from_eamil_address
        self.msg = MIMEMultipart()
        self.add_from_addr(from_name,self.__from_addr)
        
    @property
    def from_addr(self) -> str:
        '''
        邮件的发件人地址信息
        Returns
            str: 邮件发件人的信息
        '''
        return self.__from_addr
    @property
    def to_addr(self) -> list:
        '''
        Returns
            list    收件人地址list
        '''
        return self.__to_addr
    def add_to_addr(self,name:str,email:str) -> None:
        '''
        把收件人加到邮件中
        Args
            name(str):收件人名称
            email(str):收件人邮件地址

        '''
        self.__add_addr(name,email,'to')
        self.__to_addr.append(email)
    def add_cc_addr(self,name:str,email:str) -> None:
        '''
        把抄送人加到邮件中
        Args
            name(str):抄送人名称
            email(str):抄送人邮件地址

        '''
        self.__add_addr(name,email,'cc')
        self.__to_addr.append(email)
    
    def add_bcc_addr(self,name:str,email:str) ->None:
        '''
        把暗送人加到邮件中
        Args
            name(str):暗送人名称
            email(str):暗送人邮件地址

        '''
        self.__add_addr(name,email,'bcc')
        self.__to_addr.append(email)

    def add_reply_addr(self,name:str,email:str) -> None:
        '''
        把回复人加到邮件中
        Args
            name(str):回复人名称
            email(str):回复人邮件地址

        '''
        self.__add_addr(name,email,'bcc')

    def add_from_addr(self,name:str,email:str) -> None:
        '''
        把发件人加到邮件中
        Args
            name(str):发件人名称
            email(str):发件人邮件地址

        '''
        self.__add_addr(name,email,'from')

    def __add_addr(self,name:str,email:str,add_type = 'from')-> None:
        '''
        把相关人加到邮件中
        Args
            name(str):相关人名称
            email(str):相关人邮件地址
            add_type(str):相关人类型，from/to/cc/bcc/replyto

        '''
        if self.__validate_email(email):
            if str.lower(add_type) == 'to':
                self.msg['To'] = self.__format_addr('{} <{}>'.format(name,email))
            elif str.lower(add_type) == 'cc':
                self.msg['Cc'] = self.__format_addr('{} <{}>'.format(name,email))
            elif str.lower(add_type) == 'bcc':
                self.msg['Bcc'] = self.__format_addr('{} <{}>'.format(name,email))
            elif str.lower(add_type) == 'replyto':
                self.msg['Reply-To'] == self.__format_addr('{} <{}>'.format(name,email))
                self.mdg['Return-Path'] = self.__format_addr('{} <{}>'.format(name,email))
            else:
                self.msg['From'] = self.__format_addr('{} <{}>'.format(name,email))
        else:
            raise EmailNotValidError(email)

    def add_title(self,title:str) -> None:
        '''
        给邮件增加标题
        Args
            title(str):邮件标题
        '''
        self.msg['Subject'] = Header(title, 'utf-8').encode()
    def add_content(self,content:str,is_html=True) -> None:
        '''
        给邮件增加内容
        Args
            content(str):邮件内容，可以是纯文本，或者html
            is_html(bool)：plain / html的标记
        '''
        self.msg.attach(MIMEText(content,'html' if is_html else 'plain', 'utf-8'))

    def add_attachment(self,filename:str) -> None:
        '''
        给邮件增加附件
        Args
            filename(str):附件的文件名，含路径
        '''
        try:
            with open(filename, 'rb') as attachment:  
                part = MIMEBase('application', 'octet-stream')  
                part.set_payload(attachment.read())  
                encoders.encode_base64(part)  
                part.add_header('Content-Disposition', "attachment; filename= " + filename)  
                self.msg.attach(part) 
        except FileNotFoundError as e:
            error_print(e)
            
    def add_attachments(self,filenames:list) -> None:
        '''
        给邮件增加多个附件
        Args
            filenames(list):附件的文件名，含路径
        '''
        for filename in filenames:
            self.add_attachment(filename)


    def __format_addr(self,s):
        '''
        给邮件地址信息格式美化
        Args
            s(str): 按照{} <{}>传进来的邮件地址信息
        '''
        return formataddr(parseaddr(s))

    def __validate_email(self,email:str) -> bool:
        '''
        验证邮件地址
        Args
            email(str):邮件地址
        Returns
            bool:如果地址合法返回True
        '''
        return True if re.match("^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?)$", \
            email) != None else False
    def as_string(self) -> str:
        '''
        将邮件对象返回string
        '''
        return self.msg.as_string()
class AsteriskMail():

    def __init__(self,smtp_server:str,smtp_port:int,smtp_username:str,smtp_password:str) -> None:
        '''
        初始化时实例化smtp对象，并读取AppConfig.json中设置，连接服务器，并登陆
        Args:
            smtp_server(str):smtp服务器地址，域名或者ip地址
            smtp_port(int):smtp服务器的端口号
            smtp_username(str):smtp服务器用户名
            smtp_password(str):smtp服务器密码
        '''
        self.smtp_obj = smtplib.SMTP_SSL(smtp_server,smtp_port) 
        self.smtp_obj.login(smtp_username,smtp_password) 

    def sendmail(self,msg:AsteriskMailMsg) -> None:
        '''
        将email msg发送出去
        Args
            msg(MosMailMsg):需要发送的email msg实例
        '''
        try:
            self.smtp_obj.sendmail(msg.from_addr, msg.to_addr, msg.as_string())

        except EmailNotValidError as em:
            wprint('email 地址无效，邮件未发送')
            error_print(em)
        except smtplib.SMTPServerDisconnected:
            self.smtp_obj.connect()
            self.smtp_obj.sendmail(msg.from_addr, msg.to_addr, msg.as_string())
        except smtplib.SMTPException as e:
            error_print(e)

        

    def __del__(self) -> None:
        '''
        对象解构时断开smtp的连接
        '''
        if self.smtp_obj:
            self.smtp_obj.quit()
            self.smtp_obj.close
        


import json
import os
from asteriskutils.tools import wprint,error_print

if  __name__ == "__main__":
    import sys
    error_print('本段代码必须从应用主入口导入执行。退出！')
    sys.exit(1)

'''
AppConfig的默认配置，当AppConfig.json文件缺失时使用

'''
AppConfig = {
    "version":"0.1.0",
    "author":"Asterisk Dev Team",
    "email":"geoshan@163.com",
    "debug": 1,
    "prompt":"Asterisk-utils > ",
    "log":{
        "log_level":"DEBUG",
        "log_path":"{App}/log",
        "log_file_name":"%Y-%m-%d",
        "log_format":"%(asctime)s - %(levelname)s - %(message)s"
    },
    "logo_text_file":"logo.attpl",
    "title_text_file":"title.attpl",
    
}
# 读取系统默认参数，主要为：version、debug、log等
try:
    with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'asterisk.json'),'r',encoding='utf8') as fp:
        AppConfig = json.load(fp)
    with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'title.attpl'),"r",encoding='utf8') as fp:
        AppConfig['title_text'] = fp.read()
    with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'logo.attpl'),"r",encoding='utf8') as fp:
        AppConfig['logo_text'] = fp.read()
    
except FileNotFoundError:
    wprint('当前目录下的配置文件asterisk.json不存在，启用默认配置。')
def get_lang_pack(locale:str = None) -> dict:
    '''
    获取语言包
    Returns:
        dict:语言包
    '''
    try:
        if locale:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), f'lang/{locale.lower()}.json'),"r",encoding='utf8') as lang_fp:
                return json.load(lang_fp)
        else:            
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), f'lang/{AppConfig["lang"].lower()}.json'),"r",encoding='utf8') as lang_fp:        
                return json.load(lang_fp)
    except FileNotFoundError:
        wprint('当前目录下的配置文件语言包不存在，启用默认配置。')
        with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'lang/zh-cn.json'),"r",encoding='utf8') as lang_fp:
            return json.load(lang_fp)
lang_pack = {}







from setuptools import setup,find_packages
import time,json

'''
只提供setup.py文件，setup.cfg文件将在后续版本提供
使用以下命令生成whl文件：
python3.12 -m build
使用以下命令上传pypi
python3.12 -m twine upload dist/*

python3 -m twine upload --repository testpypi dist/*
注意：需要提前安装wheel包
'''

with open('src/asteriskutils/asterisk.json','r',encoding='utf8') as fp:
    setting = json.load(fp)

build_no = time.strftime('%m%d.%H%M',time.localtime())
# 该版本号将在后续版本中以package的属性实现，以便于setup.cfg文件方便使用

with open('README.md','r',encoding='utf-8') as fp:
    long_description = fp.read()

setup(
    name="asterisk-utils",  #pypi中的名称，pip或者easy_install安装时使用的名称
    version="{}.{}".format(setting['version'],build_no),
    author="Shan,Tian",
    author_email="geoshan@163.com",
    description=("Utility tools for asterisk series."),
    license="Apache License 2.0",
    keywords="tools,email,print with log,asterisk",
    packages=find_packages(where="src/",exclude=['build','dist','*egg*']),  # 需要打包的目录列表，排除测试项目以及build之后产生的目录
    package_dir={'': 'src'},  # 包对应的目录，""表示src目录

    # 需要安装的依赖
    install_requires=[
        'colorama>=0.4.4',
        'Deprecated >=1.2.13'
    ],
    include_package_data=True,


    package_data={
        # If any package contains *.json or *.attpl or *.config files, include them:
        "": ["*.json", "*.attpl","*.config"]
    },

    # Python版本的要求
    python_requires='>=3.10.0',

    platforms='Independent',

    # 添加这个选项，是为了在安装后，可以使用命令行自动生成项目文件和目录，以及目录下的默认配置文件等
    # 注意：模块与函数之间是冒号:
    # entry_points={'console_scripts': [
    #     'asencrypt = asterisksecurity.console:encrypt',
    #     'asdecrypt = asterisksecurity.console:decrypt',
    # ]},

    long_description=long_description, # 本段代码将在编辑readme后使用
    long_description_content_type='text/markdown',
    classifiers=[  # 程序的所属分类列表
        "Development Status :: 4 - Beta",
        "Topic :: Utilities",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.10"
    ],
    # 此项需要，否则卸载时报windows error
    zip_safe=False
)